/* ============================================================================================================================================= /
 * PROJECT:         Orologio GLCD
 * FILE:            Main.h
 * LAST REV:        2.4
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F4550 su FREEDOM 2
 * CREATED:         01/12/2013
 * UPDATED:         13/05/2014
 *
 * DESCRIP:         Settings
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 2.4              13/05/2014          - Inserito offset LM35 per Freedom
 * 2.3              12/04/2014          - Inserite posizioni scritta trashold
 * 2.2              11/04/2014          - Inserita variante di selezione hardware
 * 2.0              07/03/2014          - Disposizione sorgente migliorata
 * 1.1              04/12/2013          - Prima release
 * 1.0              01/12/2013          - Inizio sviluppo

/* I/O PIC18F4550 ============================================================================================================================== /
 * RA0:     Ingresso analogico sensore luce
 * RA1:     NC
 * RA2:     Ingresso analogico sensore temperatura
 * RA3:     GLCD_DI
 * RA4:     GLCD_RW
 * RA5:     GLCD_E
 * RB0:     I2C DATA
 * RB1:     I2C CLK
 * RB2:     Segnale interrupt da RTCC
 * RB4:     SW1: Retroilluminazione/Down
 * RB5:     SW2: Stop allarme/Up
 * RB6:     SW3: Setup allarme
 * RB7:     SW4  Setup data e ora
 * RC0:     Buzzer
 * RC1:     LCD retroilluminazione (PWM)
 * PORTD:   Data I/O GLCD
 * RE0:     GLCD_CS1
 * RE1:     GLCD_CS2
 * RE2:     GLCD_RST
 *
/* ============================================================================================================================================ */

/* CONFIGURATION BITS ========================================================================================================================= */
#pragma config PBADEN   = OFF                                                           // PORTB<4:0> come digital I/O
#pragma config XINST    = OFF                                                           // Set istruzioni estese abilitato
#pragma config DEBUG    = ON                                                            // Debugger Interno (ICD) Disabilitato, RB6 e RB7 = GPIO
#pragma config LVP      = OFF                                                           // Low voltage program Disablitata
#pragma config FOSC     = HS                                                            // Oscillatore interno (IRCF Osc=8MHz)
#pragma config PLLDIV   = 1                                                             // PLL no prescale
#pragma config CPUDIV   = OSC1_PLL2                                                     // No post scale
#pragma config PWRT     = ON                                                            // PWRT Abilitato
#pragma config BOR      = OFF                                                           // BOR Disabilitato
#pragma config MCLRE    = ON                                                            // MCLR Abiltato
#pragma config WDT      = OFF                                                           // Disabilitato e controllato da SWDTEN
#pragma config STVREN   = ON                                                            // Restet da Stak uder/over flow abilitato
#pragma config CPD      = OFF                                                           // Protezione dati in EEPROM Disabilitato
#pragma config WRT0     = OFF                                                           // block 0 non protetto
#pragma config WRT1     = OFF                                                           // block 1 non protetto
#pragma config WRTB     = OFF                                                           // Write protect bootblock (ON/OFF)
#pragma config WRTC     = OFF                                                           // Protezione del registro di Configurazione disabilitata

/* CONFIGURAZIONE HW ========================================================================================================================== */
#define FREEDOM
//#define DEDICATED_HW

/* CONSTANTI ==================================================================================================================================== */
#define XTAL_FREQ           20                                                          // definisco la frequenza per Libreria Delay

#ifdef FREEDOM
    #define BUZZ                LATCbits.LATC0
    #define ALARM_INT           PORTBbits.RB2                                               // interrupt allarme da RTCC
    #define SW1                 (char *)&PORTB, 0b00010000                                  // puntatore con valore l'indirizzo di PORTA, maschera bit 1 -> RA0
    #define SW2                 (char *)&PORTB, 0b00100000                                  // puntatore con valore l'indirizzo di PORTA, maschera bit 2 -> RA1
    #define SW3                 (char *)&PORTB, 0b01000000                                  // puntatore con valore l'indirizzo di PORTA, maschera bit 1 -> RA0
    #define SW4                 (char *)&PORTB, 0b10000000                                  // puntatore con valore l'indirizzo di PORTA, maschera bit 2 -> RA1
#endif

#ifdef DEDICATED_HW
    #define BUZZ                LATCbits.LATC2
    #define ALARM_INT           PORTBbits.RB2                                               // interrupt allarme da RTCC
    #define SW1                 (char *)&PORTB, 0b00001000
    #define SW2                 (char *)&PORTB, 0b00010000
    #define SW3                 (char *)&PORTB, 0b00100000
    #define SW4                 (char *)&PORTC, 0b00000001
#endif

//#define ON                  1
//#define OFF                 0

#define DUTY_LOW_NIGHT      2                                                           // valori duty cicle retroilluminazione
#define DUTY_LOW_DAY        50
#define DUTY_HIGH_NIGHT     30
#define DUTY_HIGH_DAY       200

#define BACK_ON             5                                                           // secondi in cui la retroilluminazione resta accesa

#ifdef FREEDOM
    #define LM35_OFFSET         3                                                          // offset temperatura in BCD
#endif

#ifdef DEDICATED_HW
    #define LM35_OFFSET         77                                                          // offset temperatura in BCD
#endif


/* POSIZIONI SCRITTE ============================================================================================================================ */
#define TIME_START_X        15
#define TIME_START_Y        35
#define DATE_START_X        6
#define DATE_START_Y        60
#define DAY_START_X         6
#define DAY_START_Y         50
#define ALARM_TIME_START_X  15
#define ALARM_TIME_START_Y  35
#define ALARM_BY_DAY_START_X 20
#define ALARM_BY_DAY_START_Y 48
#define ALARM_WRITE_X       100
#define ALARM_WRITE_Y       43
#define ALARM_ON_X          100
#define ALARM_ON_Y          49
#define TONE_X              100
#define TONE_Y              55
#define TONE_ON_X           100
#define TONE_ON_Y           61
#define TEMP_X              89                                                          // riga temperatura
#define TEMP_Y              60                                                          // prima colonna temperatura
#define TRESHOLD_X          50
#define TRESHOLD_Y          30

/* GIORNI ======================================================================================================================================= */
#define D0                  "Domenica "
#define D1                  "Lunedi"
#define D2                  "Martedi"
#define D3                  "Mercoledi"
#define D4                  "Giovedi"
#define D5                  "Venerdi"
#define D6                  "Sabato"

/* RTCC VALORI MINIMI E MASSIMI ================================================================================================================ */
#define MIN_SEC             0x00                                                        // valori minimi e massimi per ogni dato
#define MAX_SEC             0x59
#define MIN_MIN             0x00
#define MAX_MIN             0x59
#define MIN_HOU             0x00
#define MAX_HOU             0x23
#define MIN_DAY             0x01
#define MAX_DAY             0x31
#define MIN_MONTH           0x01
#define MAX_MONTH           0x12
#define MIN_YEAR            0x00
#define MAX_YEAR            0x99
#define MAX_DAY_WEEK        0x06

/* ============================================================================================================================================= /
 * FUNCTION:        DefineIO
 * Overview:        Definizione e inizializzazione ingressi e uscite
/* ============================================================================================================================================ */

void Define_IO(void) {

#ifdef FREEDOM
    LATA =      0b00000000;                                                             // Reset Latch uscite
    TRISA =     0b00000101;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    TRISB =     0b11110101;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    TRISC =     0b11000000;
    LATD =      0b00000000;                                                             // Reset Latch uscite
    TRISD =     0b00000000;
    LATE = 0x00;
    TRISE = 0xF8;
#endif

#ifdef DEDICATED_HW
    LATA =      0b00000000;                                                             // Reset Latch uscite
    TRISA =     0b00000101;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    TRISB =     0b11111101;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    TRISC =     0b11000001;
    LATD =      0b00000000;                                                             // Reset Latch uscite
    TRISD =     0b00000000;
    LATE =      0b00000000;
    TRISE =     0b11111000;
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:        Module_settings
 * Overview:        Configurazioni dei moduli interni
/* ============================================================================================================================================ */

void Module_settings(void) {
    UCON =      0;                                                                      // disabilito USB
    UCFG =      1;

    INTCON2bits.RBPU = 0;                                                               // abilito resistenze pull up su PORTB

    CCP1CONbits.CCP1M0 = 0;                                                             // disabilito tutti i comparatori
    CCP1CONbits.CCP1M1 = 0;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M3 = 0;

    CCP2CONbits.CCP2M0 = 0;
    CCP2CONbits.CCP2M1 = 0;
    CCP2CONbits.CCP2M2 = 0;
    CCP2CONbits.CCP2M3 = 0;

    INTCONbits.GIE = 1;                                                                 // abilito global interrupt
    INTCONbits.PEIE = 1;                                                                // abilito interrupt periferiche
    INTCONbits.TMR0IE = 1;                                                              // abilito interrupt timer 0
    PIE1bits.TMR1IE = 1;                                                                // abilito interrupt timer 0
    INTCON3bits.INT2IE = 1;                                                             // abilito interrupt INT0
    INTCON2bits.INTEDG2 = 0;                                                            // INT2 su falling edge

    // TIMER 0, Configurazioni per avere 1 interrupt ogni 250ms = (65536-preload)*(4/fosc)*prescaler = (65536-26476)*(4/20000000)*32
    T0CONbits.T0PS0 =0;                                                                 // prescaler 1/32
    T0CONbits.T0PS1 =0;                                                                 //
    T0CONbits.T0PS2 =1;
    T0CONbits.PSA = 0;                                                                  // prescaler assegnato
    T0CONbits.T0CS = 0;                                                                 // da clock interno fosc/4
    T0CONbits.T08BIT = 0;                                                               // 16 BIT
    TMR0 = 26476;                                                                       // preload timer 0

    T0CONbits.TMR0ON = 0;

    // TIMER 1, Configurazioni per avere 1 interrupt ogni 100ms = (65536-preload)*(4/fosc)*prescaler = (65536-3038)*(4/20000000)*8
    T1CONbits.T1CKPS0 = 1;                                                              // prescaler 1/8
    T1CONbits.T1CKPS1 = 1;
    T1CONbits.TMR1CS = 0;                                                               // da clock interno fosc/4
    T1CONbits.RD16 = 1;                                                                 // 16 BIT
    TMR1 = 3038;                                                                        // preload timer 1

    T1CONbits.TMR1ON = 0;

    // Configurazioni per ADC
    ADCON1 =    0b00001100; //0b00001101                                                // Abilito AN0-AN1-AN2 come ingressi analogici VREF sono impostate a massa e VCC
    ADCON2 =    0b10110100;                                                             // TAD : FOSC/4, TACQ: 16 TAD
    ADCON0bits.ADON = 0;                                                                // disabilito l' ADC

    // Configurazioni per PWM

    T2CONbits.TMR2ON = 1;
    T2CONbits.T2CKPS0 = 0;                                                              // prescaler 16
    T2CONbits.T2CKPS1 = 1;
    T2CONbits.T2OUTPS0 = 1;
    T2CONbits.T2OUTPS1 = 1;
    T2CONbits.T2OUTPS2 = 1;
    T2CONbits.T2OUTPS3 = 1;
    PR2 = 249;                                                                          // imposto il periodo

    CCP2CONbits.CCP2M = 1;

    CCP2CONbits.CCP2M2 = 1;                                                             //attivo PWM2
    CCP2CONbits.CCP2M3 = 1;
}

