/* ============================================================================================================================================= /
 * LIBRARY:         I2C initialization
 * FILE:            I2C_init.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/03/2011
 * UPDATED:         17/03/2011
 *
 * DESCRIP:         Inizializzazione protocollo i2c
 *
 * PRECONDITIONS:   - definire la frequenza del clock in MHz XTAL_FREQ
 *                  - inserire nelle Include Directory del compilatore C:\Program Files (x86)\Microchip\xc8\v1.21\include\plib
 * FUNCTIONS:       - I2C_init()
 *
 * /* ============================================================================================================================================= /
 * Change History:
 * 1.0              17/03/2014          - Prima release
/* ============================================================================================================================================ */

/* INCLUDES =================================================================================================================================== */
#include "I2C_init.h"
#include <i2c.h>

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_init
 * Input:       baud_rate: indicare il badrate espresso in KHz (unsigned int)
 *
 * Overview:    Imposta il protocollo I2C e calcola SSPADD. per ora gestito solo come master
/* ============================================================================================================================================ */
void I2C_init_master(unsigned int _baud_rate) {
    OpenI2C(MASTER, SLEW_ON);                                                           // i2c master
    SSPADD = (((XTAL_FREQ * 1000)/4)/_baud_rate)-1;                                     // calcolo SSPADD
}

