/* ============================================================================================================================================= /
 * LIBRARY:         Gestione GLCD
 * FILE:            GLCD_KS0108B.h
 * LAST REV:        1.2
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         03/01/2014
 * UPDATED:
 *
 * DEPENDENCE:      Richiede libreria Delay.c
 *                  definire la frequenza del clock utilizzato #define XTAL_FREQ
 * DESCRIP:         Settaggi di display GLCD 128*64
/* ============================================================================================================================================ */

//#include "delay.c"

// font e immagini inclusi nel progetto, commentare i font non utilizzati
#include    "Font_4x5.h"
//#include    "Font_5x7.h"
#include    "Font_7x7.h"
//#include    "Font_9x15.h"
//#include    "Font_12x19.h"
//#include    "Font_13x24.h"
//#include    "Font_18x26.h"
#include    "Font_20x31.h"
//#include    "Logo_1.h"

/* CONFIGURAZIONE HW ========================================================================================================================== */
//#define FREEDOM
//#define DEDICATED_HW

/* COSTANTI PIN =============================================================================================================================== */
#ifdef FREEDOM
#define GLCD_DATA_WRITE     LATD
#define GLCD_DATA_READ      PORTD
#define GLCD_DATA_DIR       TRISD

#define GLCD_DI             LATAbits.LATA3
#define GLCD_RW             LATAbits.LATA4
#define GLCD_E              LATAbits.LATA5
#define GLCD_CS1            LATEbits.LATE0
#define GLCD_CS2            LATEbits.LATE1
#define GLCD_RST            LATEbits.LATE2
#define GLCD_BACK_ILL       LATCbits.LATC1
#endif

#ifdef DEDICATED_HW
#define GLCD_DATA_WRITE     LATD
#define GLCD_DATA_READ      PORTD
#define GLCD_DATA_DIR       TRISD

#define GLCD_DI             LATAbits.LATA4
#define GLCD_RW             LATAbits.LATA5
#define GLCD_E              LATAbits.LATA3
#define GLCD_CS1            LATEbits.LATE2
#define GLCD_CS2            LATEbits.LATE1
#define GLCD_RST            LATEbits.LATE0
#define GLCD_BACK_ILL       LATCbits.LATC1
#endif

/* COSTANTI GLCD ============================================================================================================================== */
#define INPUT               0b11111111                                                  // costanti per direzione dati PIC I/O
#define OUTPUT              0b00000000
#define X_RESOLUTION        128
#define Y_RESOLUTION        64
#define MAX_X               64
#define MAX_PAGES           8

#define CS_ACTIVE_HIGH                                                                  // se i CS del GLCD sono attivo alto
//#define CS_ACTIVE_LOW                                                                   // se i CS del GLCD sono attivo basso

#ifdef CS_ACTIVE_HIGH
    #define ACTIVE          0b00000001
    #define DEACTIVE        0b00000000
#endif

#ifdef CS_ACTIVE_LOW
    #define ACTIVE          0b00000000
    #define DEACTIVE        0b00000001
#endif

/* COMANDI GLCD =============================================================================================================================== */
#define GLCD_ON             0b00111111
#define GLCD_OFF            0b00111110
#define GLCD_Y_PAGE         0b10111000
#define GLCD_X_ADDRESS      0b01000000
#define GLCD_START_LINE     0b11000000

/* FLAG MASK GLCD ============================================================================================================================= */
#define GLCD_BUSY_FLAG      0b10000000

/* GENERAL CONST ============================================================================================================================== */
#define HIGH                0b00000001
#define LOW                 0b00000000
#define ON                  0b00000001
#define OFF                 0b00000000
#define GLCD_EN_RESET       0b00000000
#define GLCD_DIS_RESET      0b00000001
#define STATUS              0b00000000
#define COMMAND             0b00000000
#define DATA                0b00000001

/* Impostazione delay minimo in base a XTAL_FREQ ============================================================================================== */
#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#if (XTAL_FREQ <= 20)
    #define __delay_minimo();  Nop();
#endif

#if (XTAL_FREQ > 20)
    #define __delay_minimo();  Nop(); Nop();
#endif

/* PROTOTIPI ================================================================================================================================== */
char GLCD_read(char _DI);
void GLCD_send(char _DI, char _data);
void GLCD_set_display(char _status);
void GLCD_set_page(char _page);
void GLCD_set_address(char _address);
void GLCD_wait_busy(void);
void GLCD_init(void);
void GLCD_clear(void);
void GLCD_write_byte(char _page, char _address, char _data);
void GLCD_write_bit(char _x, char _y, char _on_off);
void GLCD_drow_image(const char  *_image);
void GLCD_drow_line_horizontal (char _x_origin, char _x_dest, char _y, char _on_off);
void GLCD_drow_line_vertical (char _y_origin, char _y_dest, char _x, char _on_off);
void GLCD_drow_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off);
void GLCD_fill_block (char _x_origin, char _x_dest, char _y_origin, char _y_dest, char _on_off);
char GLCD_write_char(char _x_pos, char _y_pos, char _character, char _font_type);
void GLCD_write_string (char _x_pos, char _y_pos, const char _string[], char _font_type);
void GLCD_write_unsint(char _x_pos, char _y_pos, unsigned int _unsint, char _font_type);
void GLCD_write_sint(char _x_pos, char _y_pos, signed int _sint, char _font_type);
void GLCD_write_PBCD(char _x_pos, char _y_pos,char _pbcd, char _font_type);
void GLCD_set_font_type (char _font_type);
void GLCD_reset_font_type (void);
void GLCD_back_illum(char _on_off);